// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package house

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-household/errors"
	dbmodel "property-household/model"
	pb_v1 "property-household/pb/v1"
	"property-household/utils"
)

func checkHouseholdUserListParam(req *pb_v1.HouseholdUserListRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "id不能为空")
	}

	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	return nil
}


//
func HouseholdUserList(ctx context.Context, req *pb_v1.HouseholdUserListRequest) (reply *pb_v1.HouseholdUserListReply, err error) {

	reply = &pb_v1.HouseholdUserListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkHouseholdUserListParam(req)
	if err != nil {
		return nil, err
	}

	p := dbmodel.THouseApproved{}
	where := map[string]interface{}{}
	if req.GardenId > 0 {
		where["garden_id"] = req.GardenId
	}
	if req.Name != "" {
		where["name"] = req.Name
	}

	reply.Page = req.Page
	reply.Total, err = p.CountGroup(database.DB(), where, nil, "uid")
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := p.ListGroup(database.DB(), where, nil, "uid", int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	gardenIdsM := map[int64]bool{}
	gardenIds := []int64{}
	for _, v := range list {
		if gardenIdsM[v.GardenId] {
			continue
		}
		gardenIdsM[v.GardenId] = true
		gardenIds = append(gardenIds, v.GardenId)
	}
	gardenInfosM, err := getGardenInfos(gardenIds)
	if err != nil {
		return nil, err
	}

	reply.List = make([]*pb_v1.HouseholdUserItem, len(list))
	for i, v := range list {
		reply.List[i] = &pb_v1.HouseholdUserItem{
			Id:            v.ID,
			UserType:      v.UserType,
			GardenName:    gardenInfosM[v.GardenId].GardenName,
			IdType:v.IdType,
			IdNumber:v.IdNumber,
			Phone:v.Phone,
			Name:v.Name,
		}

		decrypt := utils.CertDecrypt(reply.List[i].IdNumber)
		if decrypt != "" {
			reply.List[i].IdNumber = decrypt
		}

	}

	return reply, nil

}
